/**
 * FLL - Level 3
 *
 * Project: FSS Read
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common data structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fss_read_common_define_h
#define _fss_read_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * fss_read_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_fss_read_allocation_d_
  #define fss_read_allocation_console_d 4
  #define fss_read_allocation_large_d   2048
  #define fss_read_allocation_pipe_d    16384
  #define fss_read_allocation_small_d   128
#endif // _di_fss_read_allocation_d_

/**
 * The program allocation block defines.
 *
 * fss_payload_read_block_*_d:
 *   - max:        The max block read size before checking for interrupt.
 *   - read_small: The block read size for small files.
 *   - read_large: The block read size for large files.
 */
#ifndef _di_fss_read_allocation_block_d_
  #define fss_read_allocation_block_max_d        16777216
  #define fss_read_allocation_block_read_small_d 8192
  #define fss_read_allocation_block_read_large_d 65536
#endif // _di_fss_read_allocation_block_d_

/**
 * Flags not passed to the main function but instead assigned by individual standards to designate how they work.
 *
 * fss_read_feature_flag_*_d:
 *   - none:               No flags set.
 *   - content_has_close:  The Content has a closing character which needs to be counted and handled when processing both Object and Content, but is otherwise not processed or printed.
 *   - content_multiple:   Designate that multiple Content is allowed for an Object for this standard rather than a single Content per Object.
 *   - depth_multiple:     Designate that multiple depths are allowed within Content rather than being single depth Content.
 *   - line_single:        Designate that only a single line for Content is allowed for an Object for this standard rather than allowing multiple lines per Content per Object.
 *   - object_align:       When printing the Object, align its Object start with the Content close (only useful if there is a Content close character).
 *   - object_as_line:     The Object is counted as its own line for the purpose of -l/--line or any other similar behavior.
 *   - object_trim:        Empty space before an after Objects are ignored while processing without affecting printing behavior.
 *   - object_trim_expand: Objects should be print all characters (generally white space) before the Object until the line start and after the Object until the line ends or the Object opener (whichever is first or most correct).
 *   - quote_content:      Designate that the standard supports quotes on Content.
 *   - quote_object:       Designate that the standard supports quotes on Objects.
 */
#ifndef _di_fss_read_feature_flag_d_
  #define fss_read_feature_flag_none_d               0x0
  #define fss_read_feature_flag_content_has_close_d  0x1
  #define fss_read_feature_flag_content_multiple_d   0x2
  #define fss_read_feature_flag_depth_multiple_d     0x4
  #define fss_read_feature_flag_line_single_d        0x8
  #define fss_read_feature_flag_object_align_d       0x10
  #define fss_read_feature_flag_object_as_line_d     0x20
  #define fss_read_feature_flag_object_trim_d        0x40
  #define fss_read_feature_flag_object_trim_expand_d 0x80
  #define fss_read_feature_flag_quote_content_d      0x100
  #define fss_read_feature_flag_quote_object_d       0x200
#endif // _di_fss_read_feature_flag_d_

/**
 * Flags passed to the main function or program.
 *
 * fss_read_main_flag_*_d:
 *   - none:                   No flags set.
 *   - at:                     The object at the given position is being selected (Think of this as select a row for some Object).
 *   - columns:                The total columns found and selected is printed instead of the Content.
 *   - content:                The Content is to be printed.
 *   - copyright:              Print copyright.
 *   - delimit:                How delimits are to be handled is designated.
 *   - depth:                  The depth is specified at least once.
 *   - empty:                  Empty Content will be printed (Objects that have no Content will have their empty Content printed).
 *   - help:                   Print help.
 *   - line:                   A specific Content at a given line is to be selected (Think of this as select a row for some Content).
 *   - name:                   A specific Object name has been requested.
 *   - object:                 The Object is to be printed.
 *   - object_content:         A helper flag representing object and content flag bits being set.
 *   - original:               Enable original printing, where the quotes are printed and no delimits are applied.
 *   - payload_create:         Create the payload Object with empty Content if the payload Object is missing (when using FSS Payload and related).
 *   - payload_error:          Treat missing or invalid payload as an error (when using FSS Payload and related).
 *   - payload_warn:           Treat missing or invalid payload as a warning (when using FSS Payload and related).
 *   - pipe:                   Use the input pipe.
 *   - pipe_format:            Print using the special pipe format.
 *   - select:                 A specific Content at a given position is to be selected (Think of this as select a column for some Content).
 *   - select_content:         A helper flag representing select and content flag bits being set.
 *   - total:                  The total lines found and selected is printed instead of the Content.
 *   - trim:                   Empty space before and after Objects and Content will not be printed (They will be trimmed).
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 */
#ifndef _di_fss_read_main_flag_d_
  #define fss_read_main_flag_none_d                   0x0
  #define fss_read_main_flag_at_d                     0x1
  #define fss_read_main_flag_columns_d                0x2
  #define fss_read_main_flag_content_d                0x4
  #define fss_read_main_flag_copyright_d              0x8
  #define fss_read_main_flag_delimit_d                0x10
  #define fss_read_main_flag_depth_d                  0x20
  #define fss_read_main_flag_empty_d                  0x40
  #define fss_read_main_flag_help_d                   0x80
  #define fss_read_main_flag_line_d                   0x100
  #define fss_read_main_flag_name_d                   0x200
  #define fss_read_main_flag_object_d                 0x400
  #define fss_read_main_flag_object_content_d         0x404
  #define fss_read_main_flag_original_d               0x800
  #define fss_read_main_flag_payload_create_d         0x1000
  #define fss_read_main_flag_payload_error_d          0x2000
  #define fss_read_main_flag_payload_warn_d           0x4000
  #define fss_read_main_flag_pipe_d                   0x8000
  #define fss_read_main_flag_pipe_format_d            0x10000
  #define fss_read_main_flag_select_d                 0x200000
  #define fss_read_main_flag_select_content_d         0x200004
  #define fss_read_main_flag_total_d                  0x400000
  #define fss_read_main_flag_trim_d                   0x800000
  #define fss_read_main_flag_version_d                0x100000
  #define fss_read_main_flag_version_copyright_help_d 0x100088
#endif // _di_fss_read_main_flag_d_

/**
 * Flags for fine-tuned print control.
 *
 * fss_read_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_fss_read_print_flag_d_
  #define fss_read_print_flag_none_d    0x0
  #define fss_read_print_flag_debug_d   0x1
  #define fss_read_print_flag_error_d   0x2
  #define fss_read_print_flag_file_d    0x4
  #define fss_read_print_flag_in_d      0x8
  #define fss_read_print_flag_out_d     0x10
  #define fss_read_print_flag_message_d 0x20
  #define fss_read_print_flag_warning_d 0x40
#endif // _di_fss_read_print_flag_d_

/**
 * The program signal defines.
 *
 * fss_read_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_fss_read_signal_d_
  #define fss_read_signal_check_d          500000
  #define fss_read_signal_check_failsafe_d 20000
  #define fss_read_signal_check_tiny_d     4
  #define fss_read_signal_check_short_d    16
#endif // _di_fss_read_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_read_common_define_h
