#include "fss_read.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_fss_read_process_
  void fss_read_process(void * const void_main) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    // For depth, most standards do not support nesting, so any depth greater than 0 can be predicted without processing the buffer.
    // For select, most standards do not support multiple select, so any select greater than 0 can be predicted without processing the buffer.
    if (!(main->setting.feature & fss_read_feature_flag_depth_multiple_d) && main->setting.depth.depth || !(main->setting.feature & fss_read_feature_flag_content_multiple_d) && (main->setting.flag & fss_read_main_flag_select_d) && main->setting.select) {
      if (main->setting.flag & fss_read_main_flag_total_d) {
        fss_read_print_number(&main->program.output, 0);
      }

      main->setting.state.status = F_okay;

      return;
    }

    // The process_load() callback is required to be non-NULL.
    if (!main->callback.process_load) return;

    if (main->setting.buffer.used) {
      main->setting.range.start = 0;
      main->setting.range.stop = main->setting.buffer.used;
    }
    else {
      main->setting.range.start = 1;
      main->setting.range.stop = 0;
    }

    main->setting.objects.used = 0;
    main->setting.contents.used = 0;
    main->setting.comments.used = 0;
    main->setting.delimits_object.used = 0;
    main->setting.delimits_content.used = 0;
    main->setting.nest.used = 0;
    main->setting.quotes_object.used = 0;
    main->setting.quotes_content.used = 0;

    main->callback.process_load(main);
    if (F_status_is_error(main->setting.state.status)) return;

    const f_number_unsigneds_t * const delimits_object = !(main->setting.flag & fss_read_main_flag_original_d) && fss_read_delimit_object_is(main, 0) ? &main->setting.delimits_object : &fss_read_except_none_c;
    const f_number_unsigneds_t * const delimits_content = !(main->setting.flag & fss_read_main_flag_original_d) && fss_read_delimit_content_is(main, 0) ? &main->setting.delimits_content : &fss_read_except_none_c;

    bool names[main->setting.objects.used];

    if (main->callback.process_name) {
      main->callback.process_name(main, names);
      if (F_status_is_error(main->setting.state.status)) return;
    }
    else if (main->setting.objects.used) {

      // If no processing is enabled, then default to F_true to enable all names.
      memset(names, F_true, sizeof(bool) * main->setting.objects.used);
    }

    if (main->setting.flag & fss_read_main_flag_total_d) {
      if (main->callback.process_total) {
        main->callback.process_total(main, names);
      }
    }
    else if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
      if (main->callback.process_at) {

        main->callback.process_at(main, names, *delimits_object, *delimits_content);
      }
    }
    else if (main->setting.flag & fss_read_main_flag_columns_d) {
      if (main->callback.process_columns) {
        main->callback.process_columns(main, names);
      }
    }
    else {
      f_number_unsigned_t i = 0;

      if (main->setting.flag & fss_read_main_flag_line_d) {
        if (main->callback.process_at_line) {
          f_number_unsigned_t line = 0;

          for (; i < main->setting.contents.used && main->setting.state.status != F_success; ++i) {

            if (!names[i]) continue;
            if (fss_read_signal_check(main)) return;

            main->callback.process_at_line(void_main, i, *delimits_object, *delimits_content, &line);
            if (F_status_is_error(main->setting.state.status)) return;
          } // for
        }
      }
      else if (main->callback.print_at) {
        for (; i < main->setting.contents.used; ++i) {

          if (!names[i]) continue;
          if (fss_read_signal_check(main)) return;

          main->callback.print_at(&main->program.output, i, *delimits_object, *delimits_content);
        } // for
      }
    }

    if (F_status_is_error(main->setting.state.status)) return;

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_

#ifndef _di_fss_read_process_at_
  void fss_read_process_at(void * const void_main, const bool names[], const f_number_unsigneds_t delimits_object, f_number_unsigneds_t delimits_content) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    if (main->setting.depth.value_at >= main->setting.objects.used) {
      if (main->setting.flag & (fss_read_main_flag_columns_d | fss_read_main_flag_total_d)) {
        fss_read_print_number(&main->program.output, 0);
      }

      main->setting.state.status = F_okay;

      return;
    }

    // If the standard only has one line per Content, then any line value greater than 0 equates to no line to print.
    if ((main->setting.flag & fss_read_main_flag_line_d) && (main->setting.feature & fss_read_feature_flag_line_single_d)) {
      if (main->setting.line) {
        main->setting.state.status = F_okay;

        return;
      }
    }

    f_number_unsigned_t i = 0;
    f_number_unsigned_t at = 0;

    for (; i < main->setting.objects.used; ++i) {

      if (!names[i]) continue;
      if (fss_read_signal_check(main)) return;

      if (at == main->setting.depth.value_at) {
        if (main->setting.flag & fss_read_main_flag_line_d) {

          // If using "--at" for standards that only support one line per Object, then the only valid line is line 0.
          if ((main->setting.feature & fss_read_feature_flag_line_single_d) && !at) break;

          if (main->callback.process_at_line) {
            f_number_unsigned_t line = 0;

            main->callback.process_at_line(void_main, i, delimits_object, delimits_content, &line);
          }
        }
        else if (main->setting.flag & fss_read_main_flag_columns_d) {
          fss_read_print_number(&main->program.output, main->setting.contents.array[i].used);
        }
        else if (main->callback.print_at) {
          main->callback.print_at(&main->program.output, i, delimits_object, delimits_content);
        }

        if (F_status_is_error(main->setting.state.status)) return;

        main->setting.state.status = F_okay;

        return;
      }

      ++at;
    } // for

    if (F_status_is_error(main->setting.state.status)) return;

    if (main->setting.flag & fss_read_main_flag_total_d) {
      fss_read_print_number(&main->program.output, 0);
    }

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_at_

#ifndef _di_fss_read_process_at_line_
  void fss_read_process_at_line(void * const void_main, const f_number_unsigned_t at, const f_number_unsigneds_t delimits_object, const f_number_unsigneds_t delimits_content, f_number_unsigned_t * const line) {

    if (!void_main || !line) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    // 0x1 = object printed, 0x2 = content printed, 0x4 = object end printed, 0x8 = content is empty, 0x10 = select is out of range.
    uint8_t print_state = 0x0;

    if (main->setting.flag & fss_read_main_flag_object_d) {
      if (*line == main->setting.line) {
        if (main->callback.print_object) {
          if (F_status_set_fine(main->callback.print_object(&main->program.output, at, delimits_object)) != F_output_not) {
            print_state = 0x1;
          }
        }

        if (!(main->setting.flag & fss_read_main_flag_content_d)) {
          if (main->callback.print_object_end) {
            main->callback.print_object_end(&main->program.output);
          }

          if (!(main->setting.feature & fss_read_feature_flag_object_as_line_d)) {
            if (main->callback.print_set_end) {
              main->callback.print_set_end(&main->program.output, at);
            }
          }

          main->setting.state.status = F_success;

          return;
        }

        if ((main->setting.feature & fss_read_feature_flag_object_as_line_d) && main->callback.print_object_end) {
          if (F_status_set_fine(main->callback.print_object_end(&main->program.output)) != F_output_not) {
            print_state = 0x4;
          }
        }
      }

      if ((main->setting.feature & fss_read_feature_flag_object_as_line_d) || !(main->setting.flag & fss_read_main_flag_content_d)) {
        ++(*line);
      }
    }

    if (main->setting.flag & fss_read_main_flag_content_d) {
      if (!main->setting.contents.array[at].used) {

        // Must process/count line when both Object and Content share the same line and Content is empty for single line Content.
        if ((main->setting.feature & fss_read_feature_flag_line_single_d)) {
          if (main->setting.flag & fss_read_main_flag_object_d) {
            if (!(main->setting.feature & fss_read_feature_flag_object_as_line_d)) {
              if (*line == main->setting.line) {
                if (main->callback.print_set_end) {
                  main->callback.print_set_end(&main->program.output, at);
                }

                main->setting.state.status = F_success;

                return;
              }

              ++(*line);
            }
          }
          else if (main->setting.flag & fss_read_main_flag_empty_d) {
            if (*line == main->setting.line) {
              if (main->callback.print_set_end) {
                main->callback.print_set_end(&main->program.output, at);
              }

              main->setting.state.status = F_success;

              return;
            }

            ++(*line);
          }
        }
        else {
          if (main->setting.flag & fss_read_main_flag_empty_d) {
            if (*line == main->setting.line) {
              if (main->callback.print_set_end) {
                main->callback.print_set_end(&main->program.output, at);
              }

              main->setting.state.status = F_success;

              return;
            }

            ++(*line);
          }
        }

        main->setting.state.status = F_okay;

        return;
      }

      f_number_unsigned_t i = 0;

      if (main->setting.feature & fss_read_feature_flag_line_single_d) {
        if (*line == main->setting.line) {
          if (main->setting.contents.array[at].used && main->callback.print_content && main->callback.print_object_end_content) {
            for (f_status_t printed = F_okay; ; ) {

              if (main->setting.contents.array[at].array[i].start <= main->setting.contents.array[at].array[i].stop) {
                if ((print_state & 0x1) && !(print_state & 0x4)) {
                  printed = main->callback.print_object_end_content(&main->program.output, at, main->setting.contents.array[at].array[i], (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0, delimits_content);

                  if (F_status_set_fine(printed) != F_output_not) {
                    print_state |= 0x6;
                  }
                }
                else {
                  printed = main->callback.print_content(&main->program.output, at, main->setting.contents.array[at].array[i], (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0, delimits_content);

                  if (F_status_set_fine(printed) != F_output_not) {
                    print_state |= 0x2;
                  }
                }
              }
              else {
                printed = F_output_not;
              }

              if (++i >= main->setting.contents.array[at].used) break;

              if (F_status_set_fine(printed) != F_output_not && main->callback.print_content_next) {
                main->callback.print_content_next(&main->program.output);
              }
            } // for
          }

          if (main->callback.print_set_end) {
            main->callback.print_set_end(&main->program.output, at);
          }

          main->setting.state.status = F_success;

          return;
        }

        ++(*line);
      }
      else if (main->callback.print_content && main->callback.print_object_end_content) {
        f_number_unsigned_t j = 0;
        f_number_unsigned_t k = 0;

        for (; i < main->setting.contents.array[at].used; ++i) {

          if (fss_read_signal_check(main)) return;

          main->setting.range = main->setting.contents.array[at].array[i];
          k = 0;

          for (j = main->setting.range.start; j <= main->setting.range.stop; ++j) {

            if (fss_read_signal_check(main)) return;

            if (k < main->setting.comments.used) {
              while (k < main->setting.comments.used && main->setting.comments.array[k].stop < j) ++k;

              if (k < main->setting.comments.used) {
                if (j >= main->setting.comments.array[k].start && j <= main->setting.comments.array[k].stop) {
                  j = main->setting.comments.array[k++].stop + 1;

                  if (j > main->setting.range.stop) break;
                }
              }
            }

            if (main->setting.buffer.string[j] == f_string_eol_s.string[0]) {
              if (*line == main->setting.line) {
                main->setting.range.stop = j;

                if ((print_state & 0x1) && !(print_state & 0x4)) {
                  main->callback.print_object_end_content(&main->program.output, at, main->setting.range, (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0, delimits_content);
                }
                else {
                  main->callback.print_content(&main->program.output, at, main->setting.range, (at < main->setting.quotes_content.used && i < main->setting.quotes_content.array[at].used) ? main->setting.quotes_content.array[at].array[i] : 0, delimits_content);
                }

                main->setting.state.status = F_success;

                return;
              }

              main->setting.range.start = j + 1;

              if (j <= main->setting.range.stop) {
                ++(*line);
              }
            }
          } // for
        } // for
      }

      // Simulate the ending by printing.
      if ((main->setting.flag & fss_read_main_flag_object_d) && (main->setting.feature & fss_read_feature_flag_content_has_close_d)) {
        if (*line == main->setting.line) {
          if (main->callback.print_set_end) {
            main->callback.print_set_end(&main->program.output, at);
          }

          main->setting.state.status = F_success;

          return;
        }

        ++(*line);
      }
    }

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_at_line_

#ifndef _di_fss_read_process_columns_
  void fss_read_process_columns(void * const void_main, const bool names[]) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    if (!(main->setting.flag & fss_read_main_flag_content_d)) {
      fss_read_print_number(&main->program.output, 0);

      main->setting.state.status = F_okay;

      return;
    }

    f_number_unsigned_t max = 0;

    if (main->setting.feature & fss_read_feature_flag_content_multiple_d) {
      f_number_unsigned_t i = 0;
      f_number_unsigned_t current = 0;

      for (f_number_unsigned_t at = 0; at < main->setting.contents.used; ++at) {

        if (!names[at]) continue;
        if (fss_read_signal_check(main)) return;

        if (main->setting.flag & fss_read_main_flag_empty_d) {
          if (main->setting.contents.array[at].used > max) {
            max = main->setting.contents.array[at].used;
          }
        }
        else {
          current = 0;

          for (i = 0; i < main->setting.contents.array[at].used; ++i) {

            if (fss_read_signal_check(main)) return;

            if (main->setting.contents.array[at].array[i].start <= main->setting.contents.array[at].array[i].stop) ++current;
          } // for

          if (current > max) {
            max = current;
          }
        }
      } // for
    }
    else {
      for (f_number_unsigned_t at = 0; at < main->setting.contents.used; ++at) {

        if (names[at]) {
          if (main->setting.flag & fss_read_main_flag_empty_d) {
            if (main->setting.contents.array[at].used) {
              max = 1;

              break;
            }
          }
          else {
            if (main->setting.contents.array[at].used && main->setting.contents.array[at].array[0].start <= main->setting.contents.array[at].array[0].stop) {
              max = 1;

              break;
            }
          }
        }

        if (fss_read_signal_check(main)) return;
      } // for
    }

    fss_read_print_number(&main->program.output, max);

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_columns_

#ifndef _di_fss_read_process_name_
  void fss_read_process_name(void * const void_main, bool names[]) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    if (main->setting.depth.index_name) {
      f_number_unsigned_t i = 0;

      memset(names, F_false, sizeof(bool) * main->setting.objects.used);

      if ((main->setting.flag & fss_read_main_flag_trim_d) || (main->setting.feature & fss_read_feature_flag_object_trim_d)) {
        for (i = 0; i < main->setting.objects.used; ++i) {

          if (fss_read_signal_check(main)) return;

          if (f_compare_dynamic_partial_except_trim_dynamic(main->setting.depth.value_name, main->setting.buffer, main->setting.objects.array[i], fss_read_except_none_c, main->setting.delimits_object) == F_equal_to) {
            names[i] = F_true;
          }
        } // for
      }
      else {
        for (i = 0; i < main->setting.objects.used; ++i) {

          if (fss_read_signal_check(main)) return;

          if (f_compare_dynamic_partial_except_dynamic(main->setting.depth.value_name, main->setting.buffer, main->setting.objects.array[i], fss_read_except_none_c, main->setting.delimits_object) == F_equal_to) {
            names[i] = F_true;
          }
        } // for
      }
    }
    else {
      memset(names, F_true, sizeof(bool) * main->setting.objects.used);
    }

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_name_

#ifndef _di_fss_read_process_total_
  void fss_read_process_total(void * const void_main, const bool names[]) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    f_number_unsigned_t total = 0;
    f_number_unsigned_t i = 0;

    if ((main->setting.flag & fss_read_main_flag_select_content_d) == fss_read_main_flag_select_content_d) {
      for (; i < main->setting.contents.used; ++i) {

        if (!names[i]) continue;
        if (fss_read_signal_check(main)) return;

        if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
          if (main->setting.depth.value_at != i) continue;
        }

        if (main->setting.contents.array[i].used) {
          if (main->setting.select < main->setting.contents.array[i].used) {
            if ((main->setting.flag & fss_read_main_flag_empty_d) || main->setting.contents.array[i].array[main->setting.select].start <= main->setting.contents.array[i].array[main->setting.select].stop) {
              ++total;
            }
          }
        }
        else if (!main->setting.select && (main->setting.flag & fss_read_main_flag_empty_d)) {
          ++total;
        }
      } // for
    }
    else if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
      for (f_number_unsigned_t at = 0; i < main->setting.objects.used; ++i) {

        if (!names[i]) continue;
        if (fss_read_signal_check(main)) return;

        if (at < main->setting.depth.value_at) {
          ++at;

          continue;
        }

        if (!(main->setting.flag & fss_read_main_flag_object_d) && (main->setting.flag & fss_read_main_flag_content_d)) {
          if (!(main->setting.contents.array[i].used || (main->setting.flag & fss_read_main_flag_empty_d))) continue;
        }

        total = 1;

        break;
      } // for
    }
    else {
      for (; i < main->setting.objects.used; ++i) {

        if (!names[i]) continue;
        if (fss_read_signal_check(main)) return;

        if (!(main->setting.flag & fss_read_main_flag_object_d) && (main->setting.flag & fss_read_main_flag_content_d)) {
          if (!(main->setting.contents.array[i].used || (main->setting.flag & fss_read_main_flag_empty_d))) continue;
        }

        ++total;
      } // for
    }

    fss_read_print_number(
      &main->program.output,
      main->setting.flag & fss_read_main_flag_line_d
        ? main->setting.line < total ? 1 : 0
        : total
    );

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_total_

#ifndef _di_fss_read_process_total_multiple_
  void fss_read_process_total_multiple(void * const void_main, const bool names[]) {

    if (!void_main) return;

    fss_read_main_t * const main = (fss_read_main_t *) void_main;

    f_number_unsigned_t total = 0;
    f_number_unsigned_t at = 0;

    if (main->setting.flag & fss_read_main_flag_content_d) {
      f_number_unsigned_t i = 0;
      f_number_unsigned_t j = 0;
      f_number_unsigned_t k = 0;

      f_range_t range = f_range_t_initialize;

      for (; at < main->setting.contents.used; ++at) {

        if (!names[at]) continue;
        if (fss_read_signal_check(main)) return;

        if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
          if (main->setting.depth.value_at != at) continue;
        }

        if (main->setting.flag & fss_read_main_flag_object_d) {
          if (!(main->setting.flag & fss_read_main_flag_content_d)) {
            ++total;

            continue;
          }

          if (main->setting.feature & fss_read_feature_flag_object_as_line_d) {
            ++total;
          }
        }

        if (main->setting.flag & fss_read_main_flag_content_d) {
          if (!main->setting.contents.array[at].used) continue;

          for (i = 0; i < main->setting.contents.array[at].used; ++i) {

            if (fss_read_signal_check(main)) return;

            range.start = main->setting.contents.array[at].array[i].start;
            range.stop = main->setting.contents.array[at].array[i].stop;

            // This content has no data, and so do not even check "include empty" because it cannot be counted as a line.
            if (range.start > range.stop) continue;

            for (j = range.start; j <= range.stop; ++j) {

              if (k < main->setting.comments.used) {
                while (k < main->setting.comments.used && main->setting.comments.array[k].stop < j) ++k;

                if (k < main->setting.comments.used) {
                  if (j >= main->setting.comments.array[k].start && j <= main->setting.comments.array[k].stop) {
                    j = main->setting.comments.array[k++].stop;

                    continue;
                  }
                }
              }

              if (main->setting.buffer.string[j] == f_string_eol_s.string[0]) ++total;
            } // for
          } // for

          // Treat Content as the last line even if it does not end with a new line.
          if ((main->setting.feature & fss_read_feature_flag_line_single_d) && i) {
            j = main->setting.contents.array[at].array[i - 1].stop;

            if (main->setting.buffer.string[j] != f_string_eol_s.string[0]) ++total;
          }

          if ((main->setting.flag & fss_read_main_flag_object_d) && (main->setting.feature & fss_read_feature_flag_content_has_close_d)) {
            ++total;
          }
        }
      } // for
    }
    else {
      for (; at < main->setting.objects.used; ++at) {

        if (!names[at]) continue;
        if (fss_read_signal_check(main)) return;

        if ((main->setting.flag & fss_read_main_flag_at_d) && main->setting.depth.index_at) {
          if (main->setting.depth.value_at != at) continue;
        }

        if (!(main->setting.flag & fss_read_main_flag_object_d) && (main->setting.flag & fss_read_main_flag_content_d)) {
          if (!(main->setting.contents.array[at].used || (main->setting.flag & fss_read_main_flag_empty_d))) continue;
        }

        ++total;
      } // for
    }

    fss_read_print_number(
      &main->program.output,
      main->setting.flag & fss_read_main_flag_line_d
        ? main->setting.line < total ? 1 : 0
        : total
    );

    main->setting.state.status = F_okay;
  }
#endif // _di_fss_read_process_total_multiple_

#ifdef __cplusplus
} // extern "C"
#endif
