#!/bin.bash
# license: lgpl-2.1-or-later
# programmer: Kevin Day
#
# Helper script for manually generating tests.
#
# This is a simple script that only accepts these arguments in this specific order:
#   1) The standard, using the 4-digit standard hexidecimal number (may also prepend "test-" to use 'fss_read -A').
#   2) The source file to read.
#   3) The destination directory to write to.
#

generate_main() {
  local file_source="${2}"
  local path_destination="${3}"
  local failure=0
  local test_base=
  local standard=
  local test_current=
  local test_set="${1}"

  standard=$(echo -n ${test_set} | sed -e 's|^.*-||g')

  if [[ $standard == "" ]] ; then
    echo "ERROR: No standard specified."

    return 1
  fi

  if [[ $file_source == "" ]] ; then
    echo "ERROR: No source file specified."

    return 1
  fi

  if [[ $path_destination == "" ]] ; then
    echo "ERROR: No destination path specified."

    return 1
  fi

  file_source=$(echo ${file_source} | sed -e 's|^//*|/|' -e 's|//*|/|' -e 's|/*$||')
  path_destination=$(echo ${path_destination} | sed -e 's|^//*|/|' -e 's|/*$|/|')

  if [[ ! -f ${file_source} ]] ; then
    echo "ERROR: The source file '${file_source}' either does not exist or is not a file."

    return 1
  fi

  if [[ ! -d ${path_destination} ]] ; then
    echo "ERROR: The destination path '${path_destination}' either does not exist or is not a directory."

    return 1
  fi

  test_base=$(echo ${file_source} | sed -e 's|.*/||g' -e 's|\..*$||')
  test_current="${path_destination}${test_base}-"

  if [[ ${test_set} == "0000" ]] ; then
    generate_operate_0000
  elif [[ ${test_set} == "0001" ]] ; then
    generate_operate_0001
  elif [[ ${test_set} == "0002" ]] ; then
    generate_operate_0002
  elif [[ ${test_set} == "0003" ]] ; then
    generate_operate_0003
  elif [[ ${test_set} == "0008" ]] ; then
    generate_operate_0008
  elif [[ ${test_set} == "000e" ]] ; then
    generate_operate_000e
  elif [[ ${test_set} == "test-0000" ]] ; then
    generate_operate_test_standard
  elif [[ ${test_set} == "test-0001" ]] ; then
    generate_operate_test_standard
  elif [[ ${test_set} == "test-0002" ]] ; then
    generate_operate_test_standard
  elif [[ ${test_set} == "test-0003" ]] ; then
    generate_operate_test_standard
  elif [[ ${test_set} == "test-0008" ]] ; then
    generate_operate_test_standard
  elif [[ ${test_set} == "test-000e" ]] ; then
    generate_operate_test_standard
  fi

  if [[ ${failure} -eq 1 ]] ; then
    return 1
  fi

  return 0
}

generate_operate_0000() {
  local last_command=

  generate_operate_run_0000 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0000 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0000 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0000 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0000 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0000 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0000 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0000 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0000 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0000 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0000 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0000 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0000 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0000 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0000 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0000 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0000 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0000 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0000 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0000 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0000 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0000 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0000 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0000 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0000 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0000 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0000 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0000 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0000 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0000 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0000 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0000 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0000 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0000 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0000 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0000 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0000 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0000 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0000 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0000 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0000 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0000 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0000 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0000 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0000 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0000 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0000 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0000 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0000 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0000 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0000 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0000 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0000 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0000 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0000 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0000 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0000 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0000 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0000 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0000 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0000 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0000 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0000 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0000 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0000 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0000 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0000 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0000 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0000 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0000 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0000 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0000 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0000 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0000 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0000 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0000 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0000 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0000 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0000 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0000 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0000 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0000 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0000 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0000 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0000 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0000 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0000 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0000 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0000 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0000 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0000 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0000 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0000 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0000 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0000 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0000 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0000 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0000 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0000 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0000 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0000 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0000 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0000 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0000 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0000 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0000 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0000 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0000 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0000 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0000 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0000 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0000 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0000 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0000 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0000 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0000 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0000 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0000 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет.expect &&
  generate_operate_run_0000 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0000 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0000 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0000 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0000 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0000 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0000 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0000 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0000 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0000 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0000 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0000 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0000 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0000 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0000 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0000 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0000 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0000 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0000 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0000 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0000 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0000 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0000 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0000 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0000 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0000 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0000 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0000 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0000 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0000 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0000 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0000 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0000 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0000 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0000 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0000 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0000 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0000 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0000 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0000 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0000 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0000 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0000 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0000 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0000 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0000 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0000 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0000 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0000 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0000 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0000 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0000 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0000 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0000 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0000 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0000 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0000 +n -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0000 +n -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0000 +n -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0000 +n -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0000 +n -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0000 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0000 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0000 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0000 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0000 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0000 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0000 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0000 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0000 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0000 +n -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run_0000 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0000 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0000 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0000 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0000 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0000 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0000 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0000 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0000 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0000 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0000 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0000 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0000 +n -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0000 +n -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0000 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0000 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0000 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0000 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0000 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0000 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0000 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0000 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0000 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0000 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0000 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0000 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0000 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0000 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0000 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0000 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0000 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0000 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0000 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0000 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0000 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0000 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0000 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0000 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0000 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0000 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0000 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0000 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0000 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0000 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0000 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0000 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0000 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0000 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0000 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0000 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0000 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0000 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0000 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0000 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_0001() {
  local last_command=

  generate_operate_run_0001 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0001 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0001 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0001 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0001 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0001 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0001 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0001 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0001 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0001 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0001 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0001 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0001 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0001 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0001 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0001 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0001 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0001 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0001 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0001 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0001 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0001 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0001 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0001 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0001 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0001 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0001 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0001 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0001 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0001 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0001 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0001 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0001 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0001 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0001 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0001 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0001 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0001 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0001 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0001 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0001 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0001 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0001 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0001 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0001 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0001 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0001 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0001 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0001 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0001 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0001 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0001 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0001 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0001 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0001 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0001 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0001 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0001 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0001 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0001 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0001 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0001 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0001 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0001 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0001 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0001 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0001 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0001 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0001 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0001 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0001 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0001 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0001 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0001 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0001 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0001 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0001 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0001 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0001 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0001 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0001 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0001 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0001 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0001 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0001 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0001 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0001 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0001 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0001 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0001 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0001 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0001 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0001 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0001 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0001 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0001 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0001 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0001 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0001 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0001 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0001 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0001 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0001 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0001 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0001 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0001 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0001 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0001 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0001 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0001 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0001 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0001 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0001 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0001 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0001 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0001 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0001 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0001 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run_0001 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0001 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0001 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0001 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0001 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0001 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0001 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0001 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0001 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0001 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0001 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0001 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0001 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0001 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0001 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0001 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0001 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0001 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0001 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0001 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0001 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0001 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0001 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0001 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0001 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0001 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0001 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0001 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0001 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0001 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0001 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0001 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0001 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0001 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0001 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0001 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0001 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0001 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0001 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0001 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0001 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0001 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0001 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0001 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0001 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0001 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0001 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0001 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0001 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0001 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0001 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0001 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0001 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0001 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0001 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0001 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0001 +n -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0001 +n -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0001 +n -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0001 +n -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0001 +n -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0001 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0001 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0001 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0001 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0001 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0001 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0001 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0001 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0001 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0001 +n -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run_0001 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0001 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0001 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0001 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0001 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0001 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0001 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0001 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0001 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0001 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0001 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0001 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0001 +n -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0001 +n -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0001 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0001 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0001 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0001 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0001 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0001 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0001 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0001 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0001 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0001 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0001 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0001 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0001 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0001 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0001 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0001 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0001 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0001 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0001 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0001 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0001 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0001 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0001 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0001 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0001 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0001 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0001 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0001 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0001 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0001 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0001 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0001 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0001 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0001 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0001 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0001 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0001 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0001 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0001 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0001 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_0002() {
  local last_command=

  generate_operate_run_0002 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0002 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0002 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0002 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0002 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0002 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0002 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0002 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0002 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0002 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0002 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0002 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0002 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0002 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0002 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0002 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0002 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0002 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0002 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0002 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0002 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0002 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0002 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0002 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0002 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0002 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0002 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0002 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0002 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0002 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0002 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0002 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0002 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0002 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0002 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0002 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0002 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0002 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0002 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0002 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0002 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0002 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0002 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0002 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0002 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0002 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0002 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0002 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0002 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0002 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0002 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0002 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0002 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0002 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0002 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0002 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0002 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0002 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0002 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0002 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0002 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0002 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0002 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0002 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0002 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0002 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0002 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0002 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0002 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0002 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0002 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0002 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0002 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0002 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0002 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0002 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0002 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0002 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0002 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0002 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0002 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0002 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0002 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0002 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0002 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0002 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0002 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0002 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0002 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0002 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0002 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0002 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0002 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0002 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0002 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0002 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0002 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0002 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0002 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0002 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0002 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0002 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0002 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0002 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0002 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0002 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0002 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0002 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0002 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0002 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0002 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0002 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0002 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0002 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0002 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0002 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0002 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0002 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run_0002 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0002 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0002 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0002 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0002 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0002 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0002 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0002 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0002 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0002 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0002 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0002 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0002 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0002 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0002 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0002 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0002 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0002 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0002 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0002 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0002 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0002 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0002 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0002 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0002 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0002 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0002 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0002 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0002 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0002 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0002 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0002 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0002 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0002 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0002 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0002 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0002 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0002 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0002 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0002 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0002 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0002 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0002 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0002 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0002 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0002 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0002 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0002 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0002 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0002 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0002 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0002 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0002 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0002 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0002 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0002 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0002 +n -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0002 +n -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0002 +n -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0002 +n -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0002 +n -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0002 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0002 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0002 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0002 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0002 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0002 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0002 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0002 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0002 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0002 +n -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run_0002 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0002 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0002 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0002 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0002 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0002 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0002 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0002 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0002 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0002 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0002 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0002 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0002 +n -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0002 +n -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0002 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0002 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0002 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0002 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0002 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0002 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0002 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0002 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0002 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0002 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0002 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0002 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0002 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0002 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0002 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0002 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0002 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0002 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0002 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0002 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0002 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0002 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0002 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0002 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0002 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0002 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0002 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0002 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0002 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0002 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0002 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0002 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0002 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0002 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0002 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0002 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0002 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0002 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0002 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0002 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_0003() {
  local last_command=

  generate_operate_run_0003 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0003 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0003 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0003 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0003 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0003 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0003 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0003 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0003 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0003 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0003 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0003 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0003 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0003 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0003 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0003 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0003 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0003 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0003 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0003 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0003 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0003 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0003 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0003 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0003 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0003 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0003 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0003 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0003 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0003 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0003 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0003 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0003 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0003 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0003 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0003 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0003 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0003 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0003 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0003 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0003 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0003 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0003 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0003 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0003 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0003 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0003 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0003 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0003 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0003 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0003 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0003 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0003 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0003 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0003 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0003 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0003 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0003 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0003 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0003 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0003 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0003 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0003 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0003 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0003 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0003 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0003 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0003 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0003 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0003 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0003 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0003 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0003 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0003 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0003 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0003 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0003 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0003 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0003 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0003 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0003 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0003 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0003 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0003 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0003 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0003 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0003 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0003 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0003 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0003 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0003 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0003 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0003 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0003 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0003 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0003 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0003 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0003 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0003 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0003 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0003 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0003 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0003 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0003 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0003 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0003 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0003 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0003 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0003 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0003 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0003 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0003 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0003 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0003 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0003 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0003 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0003 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0003 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run_0003 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0003 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0003 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0003 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0003 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0003 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0003 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0003 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0003 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0003 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0003 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0003 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0003 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0003 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0003 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0003 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0003 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0003 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0003 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0003 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0003 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0003 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0003 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0003 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0003 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0003 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0003 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0003 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0003 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0003 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0003 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0003 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0003 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0003 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0003 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0003 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0003 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0003 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0003 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0003 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0003 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0003 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0003 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0003 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0003 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0003 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0003 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0003 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0003 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0003 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0003 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0003 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0003 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0003 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0003 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0003 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0003 +n -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0003 +n -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0003 +n -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0003 +n -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0003 +n -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0003 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0003 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0003 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0003 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0003 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0003 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0003 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0003 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0003 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0003 +n -ocsO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0003 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0003 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0003 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0003 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0003 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0003 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0003 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0003 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0003 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0003 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0003 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0003 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0003 +n -onO a                      ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0003 +n -onOe a                     ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0003 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0003 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0003 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0003 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0003 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0003 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0003 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0003 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0003 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0003 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0003 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0003 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0003 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0003 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0003 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0003 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0003 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0003 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0003 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0003 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0003 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0003 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0003 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0003 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0003 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0003 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0003 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0003 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0003 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0003 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0003 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0003 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0003 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0003 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0003 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0003 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0003 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0003 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0003 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0003 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_0008() {
  local last_command=

  generate_operate_run_0008 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0008 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0008 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0008 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0008 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0008 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0008 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0008 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0008 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0008 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0008 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0008 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0008 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0008 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0008 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0008 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0008 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0008 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0008 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0008 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0008 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0008 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0008 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0008 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0008 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0008 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0008 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0008 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0008 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0008 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0008 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0008 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0008 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0008 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0008 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0008 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0008 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0008 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0008 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0008 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0008 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0008 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0008 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0008 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0008 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0008 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0008 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0008 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0008 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0008 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0008 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0008 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0008 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0008 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0008 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0008 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0008 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0008 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0008 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0008 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0008 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0008 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0008 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0008 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0008 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0008 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0008 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0008 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0008 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0008 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0008 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0008 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0008 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0008 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0008 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0008 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0008 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0008 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0008 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0008 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0008 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0008 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0008 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0008 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0008 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0008 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0008 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0008 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0008 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0008 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0008 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0008 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run_0008 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0008 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0008 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0008 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0008 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0008 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0008 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0008 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0008 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0008 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0008 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0008 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0008 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0008 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0008 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0008 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0008 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0008 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0008 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0008 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0008 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0008 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0008 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0008 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0008 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0008 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0008 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0008 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0008 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0008 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0008 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0008 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0008 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0008 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0008 +n -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0008 +n -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0008 +n -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0008 +n -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0008 +n -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0008 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0008 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0008 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0008 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0008 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0008 +n -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run_0008 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0008 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0008 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0008 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0008 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0008 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0008 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0008 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0008 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0008 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0008 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0008 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0008 +n -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0008 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0008 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0008 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0008 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0008 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0008 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0008 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0008 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0008 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0008 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0008 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0008 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0008 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0008 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0008 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0008 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0008 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0008 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  generate_operate_0008_special_cases &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_0008_special_cases() {

  if [[ ${test_base} == "test-0004-nested_simple" ]] ; then
    generate_operate_run_0008 +n -c -d 0                     ${file_source} > ${test_current}content-depth_0.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1                ${file_source} > ${test_current}content-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ct -d 0                     ${file_source} > ${test_current}content-total-depth_0.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1                ${file_source} > ${test_current}content-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 0                     ${file_source} > ${test_current}content-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1                ${file_source} > ${test_current}content-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 0                     ${file_source} > ${test_current}content-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1                ${file_source} > ${test_current}content-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -d 0 -a 0                     ${file_source} > ${test_current}content-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ct -d 0 -a 0                     ${file_source} > ${test_current}content-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0                     ${file_source} > ${test_current}content-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0                     ${file_source} > ${test_current}content-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 0                     ${file_source} > ${test_current}object-depth_0.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1                ${file_source} > ${test_current}object-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ot -d 0                     ${file_source} > ${test_current}object-total-depth_0.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1                ${file_source} > ${test_current}object-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 0                     ${file_source} > ${test_current}object-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1                ${file_source} > ${test_current}object-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 0                     ${file_source} > ${test_current}object-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1                ${file_source} > ${test_current}object-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 0 -a 0                     ${file_source} > ${test_current}object-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ot -d 0 -a 0                     ${file_source} > ${test_current}object-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0                     ${file_source} > ${test_current}object-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0                     ${file_source} > ${test_current}object-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -d 0                     ${file_source} > ${test_current}object_content-depth_0.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1                ${file_source} > ${test_current}object_content-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oct -d 0                     ${file_source} > ${test_current}object_content-total-depth_0.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1                ${file_source} > ${test_current}object_content-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 0                     ${file_source} > ${test_current}object_content-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1                ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 0                     ${file_source} > ${test_current}object_content-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1                ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -d 0 -a 0                     ${file_source} > ${test_current}object_content-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oct -d 0 -a 0                     ${file_source} > ${test_current}object_content-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0                     ${file_source} > ${test_current}object_content-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0                     ${file_source} > ${test_current}object_content-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect

    if [[ $? -ne 0 ]] ; then return 1; fi
  fi

  return 0
}

generate_operate_000e() {
  local last_command=

  generate_operate_run_0008 +n -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run_0008 +n -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run_0008 +n -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run_0008 +n -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run_0008 +n -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run_0008 +n -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run_0008 +n -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run_0008 +n -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run_0008 +n -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run_0008 +n -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run_0008 +n -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run_0008 +n -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run_0008 +n -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run_0008 +n -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run_0008 +n -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run_0008 +n -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run_0008 +n -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run_0008 +n -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run_0008 +n -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run_0008 +n -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run_0008 +n -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run_0008 +n -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run_0008 +n -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run_0008 +n -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run_0008 +n -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run_0008 +n -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run_0008 +n -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run_0008 +n -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run_0008 +n -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run_0008 +n -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run_0008 +n -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run_0008 +n -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run_0008 +n -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run_0008 +n -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run_0008 +n -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run_0008 +n -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run_0008 +n -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run_0008 +n -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run_0008 +n -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run_0008 +n -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run_0008 +n -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run_0008 +n -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run_0008 +n -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run_0008 +n -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run_0008 +n -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run_0008 +n -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run_0008 +n -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run_0008 +n -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run_0008 +n -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run_0008 +n -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run_0008 +n -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run_0008 +n -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run_0008 +n -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0008 +n -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0008 +n -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run_0008 +n -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run_0008 +n -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0008 +n -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run_0008 +n -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run_0008 +n -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run_0008 +n -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run_0008 +n -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run_0008 +n -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run_0008 +n -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run_0008 +n -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run_0008 +n -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run_0008 +n -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run_0008 +n -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run_0008 +n -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run_0008 +n -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run_0008 +n -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run_0008 +n -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run_0008 +n -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run_0008 +n -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run_0008 +n -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run_0008 +n -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run_0008 +n -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run_0008 +n -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run_0008 +n -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run_0008 +n -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run_0008 +n -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run_0008 +n -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run_0008 +n -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run_0008 +n -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run_0008 +n -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run_0008 +n -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run_0008 +n -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run_0008 +n -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run_0008 +n -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run_0008 +n -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run_0008 +n -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run_0008 +n -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run_0008 +n -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run_0008 +n -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run_0008 +n -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run_0008 +n -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run_0008 +n -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run_0008 +n -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run_0008 +n -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run_0008 +n -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run_0008 +n -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run_0008 +n -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run_0008 +n -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run_0008 +n -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run_0008 +n -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run_0008 +n -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run_0008 +n -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run_0008 +n -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run_0008 +n -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -ons привет 0              ${file_source} > ${test_current}object-name-привет-select-0.expect &&
  generate_operate_run_0008 +n -ons привет 1              ${file_source} > ${test_current}object-name-привет-select-1.expect &&
  generate_operate_run_0008 +n -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run_0008 +n -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run_0008 +n -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run_0008 +n -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run_0008 +n -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run_0008 +n -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run_0008 +n -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run_0008 +n -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run_0008 +n -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run_0008 +n -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run_0008 +n -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run_0008 +n -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run_0008 +n -clO 0                      ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run_0008 +n -clO 1                      ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run_0008 +n -clO 5                      ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run_0008 +n -clO 6                      ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run_0008 +n -clO 100                    ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run_0008 +n -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run_0008 +n -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run_0008 +n -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run_0008 +n -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run_0008 +n -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run_0008 +n -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run_0008 +n -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run_0008 +n -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run_0008 +n -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run_0008 +n -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run_0008 +n -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run_0008 +n -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run_0008 +n -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run_0008 +n -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run_0008 +n -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run_0008 +n -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run_0008 +n -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run_0008 +n -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run_0008 +n -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run_0008 +n -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run_0008 +n -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run_0008 +n -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run_0008 +n -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run_0008 +n -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run_0008 +n -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run_0008 +n -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run_0008 +n -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run_0008 +n -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run_0008 +n -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run_0008 +n -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run_0008 +n -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run_0008 +n -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run_0008 +n -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run_0008 +n -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run_0008 +n -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run_0008 +n -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run_0008 +n -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run_0008 +n -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run_0008 +n -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run_0008 +n -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run_0008 +n -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run_0008 +n -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run_0008 +n -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run_0008 +n -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run_0008 +n -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run_0008 +n -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run_0008 +n -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run_0008 +n -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run_0008 +n -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run_0008 +n -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run_0008 +n -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run_0008 +n -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run_0008 +n -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run_0008 +n -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run_0008 +n -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run_0008 +n -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run_0008 +n -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run_0008 +n -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run_0008 +n -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run_0008 +n -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_run() {
  last_command="fss_read ${@}"

  fss_read "$@"

  return $?
}

generate_operate_run_0000() {
  last_command="fss_basic_read ${@}"

  fss_basic_read "$@"

  return $?
}

generate_operate_run_0001() {
  last_command="fss_extended_read ${@}"

  fss_extended_read "$@"

  return $?
}

generate_operate_run_0002() {
  last_command="fss_basic_list_read ${@}"

  fss_basic_list_read "$@"

  return $?
}

generate_operate_run_0003() {
  last_command="fss_extended_list_read ${@}"

  fss_extended_list_read "$@"

  return $?
}

generate_operate_run_0008() {
  last_command="fss_embedded_list_read ${@}"

  fss_embedded_list_read "$@"

  return $?
}

generate_operate_run_000e() {
  last_command="fss_payload_read ${@}"

  fss_payload_read "$@"

  return $?
}

generate_operate_test_standard() {
  local last_command=

  generate_operate_run +n -A ${standard} -c                         ${file_source} > ${test_current}content.expect &&
  generate_operate_run +n -A ${standard} -cO                        ${file_source} > ${test_current}content-original.expect &&
  generate_operate_run +n -A ${standard} -ct                        ${file_source} > ${test_current}content-total.expect &&
  generate_operate_run +n -A ${standard} -cT                        ${file_source} > ${test_current}content-trim.expect &&
  generate_operate_run +n -A ${standard} -cl 0                      ${file_source} > ${test_current}content-line-0.expect &&
  generate_operate_run +n -A ${standard} -cl 1                      ${file_source} > ${test_current}content-line-1.expect &&
  generate_operate_run +n -A ${standard} -cl 5                      ${file_source} > ${test_current}content-line-5.expect &&
  generate_operate_run +n -A ${standard} -cl 6                      ${file_source} > ${test_current}content-line-6.expect &&
  generate_operate_run +n -A ${standard} -cl 100                    ${file_source} > ${test_current}content-line-100.expect &&
  generate_operate_run +n -A ${standard} -clO 0                     ${file_source} > ${test_current}content-line-0-original.expect &&
  generate_operate_run +n -A ${standard} -clO 1                     ${file_source} > ${test_current}content-line-1-original.expect &&
  generate_operate_run +n -A ${standard} -clO 5                     ${file_source} > ${test_current}content-line-5-original.expect &&
  generate_operate_run +n -A ${standard} -clO 6                     ${file_source} > ${test_current}content-line-6-original.expect &&
  generate_operate_run +n -A ${standard} -clO 100                   ${file_source} > ${test_current}content-line-100-original.expect &&
  generate_operate_run +n -A ${standard} -cs 0                      ${file_source} > ${test_current}content-select-0.expect &&
  generate_operate_run +n -A ${standard} -cs 1                      ${file_source} > ${test_current}content-select-1.expect &&
  generate_operate_run +n -A ${standard} -cs 5                      ${file_source} > ${test_current}content-select-5.expect &&
  generate_operate_run +n -A ${standard} -cs 6                      ${file_source} > ${test_current}content-select-6.expect &&
  generate_operate_run +n -A ${standard} -cs 100                    ${file_source} > ${test_current}content-select-100.expect &&
  generate_operate_run +n -A ${standard} -csO 0                     ${file_source} > ${test_current}content-select-0-original.expect &&
  generate_operate_run +n -A ${standard} -csO 1                     ${file_source} > ${test_current}content-select-1-original.expect &&
  generate_operate_run +n -A ${standard} -csO 5                     ${file_source} > ${test_current}content-select-5-original.expect &&
  generate_operate_run +n -A ${standard} -csO 6                     ${file_source} > ${test_current}content-select-6-original.expect &&
  generate_operate_run +n -A ${standard} -csO 100                   ${file_source} > ${test_current}content-select-100-original.expect &&
  generate_operate_run +n -A ${standard} -cC                        ${file_source} > ${test_current}content-columns.expect &&
  generate_operate_run +n -A ${standard} -cCe                       ${file_source} > ${test_current}content-columns-empty.expect &&
  generate_operate_run +n -A ${standard} -ca 0                      ${file_source} > ${test_current}content-at-0.expect &&
  generate_operate_run +n -A ${standard} -ca 1                      ${file_source} > ${test_current}content-at-1.expect &&
  generate_operate_run +n -A ${standard} -ca 5                      ${file_source} > ${test_current}content-at-5.expect &&
  generate_operate_run +n -A ${standard} -ca 6                      ${file_source} > ${test_current}content-at-6.expect &&
  generate_operate_run +n -A ${standard} -ca 100                    ${file_source} > ${test_current}content-at-100.expect &&
  generate_operate_run +n -A ${standard} -cn hi                     ${file_source} > ${test_current}content-name-hi.expect &&
  generate_operate_run +n -A ${standard} -cn payload                ${file_source} > ${test_current}content-name-payload.expect &&
  generate_operate_run +n -A ${standard} -cn a                      ${file_source} > ${test_current}content-name-a.expect &&
  generate_operate_run +n -A ${standard} -cnC a                     ${file_source} > ${test_current}content-name-a-columns.expect &&
  generate_operate_run +n -A ${standard} -cnCe a                    ${file_source} > ${test_current}content-name-a-columns-empty.expect &&
  generate_operate_run +n -A ${standard} -cnO a                     ${file_source} > ${test_current}content-name-a-original.expect &&
  generate_operate_run +n -A ${standard} -cnOe a                    ${file_source} > ${test_current}content-name-a-original-empty.expect &&
  generate_operate_run +n -A ${standard} -cn ""                     ${file_source} > ${test_current}content-name-.expect &&
  generate_operate_run +n -A ${standard} -cn мир                    ${file_source} > ${test_current}content-name-мир.expect &&
  generate_operate_run +n -A ${standard} -cn "привет has space"     ${file_source} > ${test_current}content-name-привет_has_space.expect &&
  generate_operate_run +n -A ${standard} -cnt hi                    ${file_source} > ${test_current}content-name-hi-total.expect &&
  generate_operate_run +n -A ${standard} -cnt payload               ${file_source} > ${test_current}content-name-payload-total.expect &&
  generate_operate_run +n -A ${standard} -cnt a                     ${file_source} > ${test_current}content-name-a-total.expect &&
  generate_operate_run +n -A ${standard} -cnt ""                    ${file_source} > ${test_current}content-name--total.expect &&
  generate_operate_run +n -A ${standard} -cnt мир                   ${file_source} > ${test_current}content-name-мир-total.expect &&
  generate_operate_run +n -A ${standard} -cnt "привет has space"    ${file_source} > ${test_current}content-name-привет_has_space-total.expect &&
  generate_operate_run +n -A ${standard} -cns hi 0                  ${file_source} > ${test_current}content-name-hi-select-0.expect &&
  generate_operate_run +n -A ${standard} -cns hi 1                  ${file_source} > ${test_current}content-name-hi-select-1.expect &&
  generate_operate_run +n -A ${standard} -cnse hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-empty.expect &&
  generate_operate_run +n -A ${standard} -cnse hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-empty.expect &&
  generate_operate_run +n -A ${standard} -cns payload 0             ${file_source} > ${test_current}content-name-payload-select-0.expect &&
  generate_operate_run +n -A ${standard} -cns payload 1             ${file_source} > ${test_current}content-name-payload-select-1.expect &&
  generate_operate_run +n -A ${standard} -cns a 0                   ${file_source} > ${test_current}content-name-a-select-0.expect &&
  generate_operate_run +n -A ${standard} -cns a 1                   ${file_source} > ${test_current}content-name-a-select-1.expect &&
  generate_operate_run +n -A ${standard} -cns a 2                   ${file_source} > ${test_current}content-name-a-select-2.expect &&
  generate_operate_run +n -A ${standard} -cns a 5                   ${file_source} > ${test_current}content-name-a-select-5.expect &&
  generate_operate_run +n -A ${standard} -cns a 6                   ${file_source} > ${test_current}content-name-a-select-6.expect &&
  generate_operate_run +n -A ${standard} -cns a 100                 ${file_source} > ${test_current}content-name-a-select-100.expect &&
  generate_operate_run +n -A ${standard} -cns "" 0                  ${file_source} > ${test_current}content-name--select-0.expect &&
  generate_operate_run +n -A ${standard} -cns "" 1                  ${file_source} > ${test_current}content-name--select-1.expect &&
  generate_operate_run +n -A ${standard} -cns мир 0                 ${file_source} > ${test_current}content-name-мир-select-0.expect &&
  generate_operate_run +n -A ${standard} -cns мир 1                 ${file_source} > ${test_current}content-name-мир-select-1.expect &&
  generate_operate_run +n -A ${standard} -cns "привет has space" 0  ${file_source} > ${test_current}content-name-привет_has_space-select-0.expect &&
  generate_operate_run +n -A ${standard} -cns "привет has space" 1  ${file_source} > ${test_current}content-name-привет_has_space-select-1.expect &&
  generate_operate_run +n -A ${standard} -cnset hi 0                ${file_source} > ${test_current}content-name-hi-select-0-empty-total.expect &&
  generate_operate_run +n -A ${standard} -cnset hi 1                ${file_source} > ${test_current}content-name-hi-select-1-empty-total.expect &&
  generate_operate_run +n -A ${standard} -cnst hi 0                 ${file_source} > ${test_current}content-name-hi-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst hi 1                 ${file_source} > ${test_current}content-name-hi-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -cnst payload 0            ${file_source} > ${test_current}content-name-payload-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst payload 1            ${file_source} > ${test_current}content-name-payload-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 0                  ${file_source} > ${test_current}content-name-a-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 1                  ${file_source} > ${test_current}content-name-a-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 2                  ${file_source} > ${test_current}content-name-a-select-2-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 5                  ${file_source} > ${test_current}content-name-a-select-5-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 6                  ${file_source} > ${test_current}content-name-a-select-6-total.expect &&
  generate_operate_run +n -A ${standard} -cnst a 100                ${file_source} > ${test_current}content-name-a-select-100-total.expect &&
  generate_operate_run +n -A ${standard} -cnst "" 0                 ${file_source} > ${test_current}content-name--select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst "" 1                 ${file_source} > ${test_current}content-name--select-1-total.expect &&
  generate_operate_run +n -A ${standard} -cnst мир 0                ${file_source} > ${test_current}content-name-мир-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst мир 1                ${file_source} > ${test_current}content-name-мир-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -cnst "привет has space" 0 ${file_source} > ${test_current}content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -cnst "привет has space" 1 ${file_source} > ${test_current}content-name-привет_has_space-select-1-total.expect &&

  generate_operate_run +n -A ${standard} -o                         ${file_source} > ${test_current}object.expect &&
  generate_operate_run +n -A ${standard} -oO                        ${file_source} > ${test_current}object-original.expect &&
  generate_operate_run +n -A ${standard} -ot                        ${file_source} > ${test_current}object-total.expect &&
  generate_operate_run +n -A ${standard} -oT                        ${file_source} > ${test_current}object-trim.expect &&
  generate_operate_run +n -A ${standard} -ol 0                      ${file_source} > ${test_current}object-line-0.expect &&
  generate_operate_run +n -A ${standard} -ol 1                      ${file_source} > ${test_current}object-line-1.expect &&
  generate_operate_run +n -A ${standard} -ol 5                      ${file_source} > ${test_current}object-line-5.expect &&
  generate_operate_run +n -A ${standard} -ol 6                      ${file_source} > ${test_current}object-line-6.expect &&
  generate_operate_run +n -A ${standard} -ol 100                    ${file_source} > ${test_current}object-line-100.expect &&
  generate_operate_run +n -A ${standard} -olO 0                     ${file_source} > ${test_current}object-line-0-original.expect &&
  generate_operate_run +n -A ${standard} -olO 1                     ${file_source} > ${test_current}object-line-1-original.expect &&
  generate_operate_run +n -A ${standard} -olO 5                     ${file_source} > ${test_current}object-line-5-original.expect &&
  generate_operate_run +n -A ${standard} -olO 6                     ${file_source} > ${test_current}object-line-6-original.expect &&
  generate_operate_run +n -A ${standard} -olO 100                   ${file_source} > ${test_current}object-line-100-original.expect &&
  generate_operate_run +n -A ${standard} -os 0                      ${file_source} > ${test_current}object-select-0.expect &&
  generate_operate_run +n -A ${standard} -os 1                      ${file_source} > ${test_current}object-select-1.expect &&
  generate_operate_run +n -A ${standard} -os 5                      ${file_source} > ${test_current}object-select-5.expect &&
  generate_operate_run +n -A ${standard} -os 6                      ${file_source} > ${test_current}object-select-6.expect &&
  generate_operate_run +n -A ${standard} -os 100                    ${file_source} > ${test_current}object-select-100.expect &&
  generate_operate_run +n -A ${standard} -osO 0                     ${file_source} > ${test_current}object-select-0-original.expect &&
  generate_operate_run +n -A ${standard} -osO 1                     ${file_source} > ${test_current}object-select-1-original.expect &&
  generate_operate_run +n -A ${standard} -osO 5                     ${file_source} > ${test_current}object-select-5-original.expect &&
  generate_operate_run +n -A ${standard} -osO 6                     ${file_source} > ${test_current}object-select-6-original.expect &&
  generate_operate_run +n -A ${standard} -osO 100                   ${file_source} > ${test_current}object-select-100-original.expect &&
  generate_operate_run +n -A ${standard} -oC                        ${file_source} > ${test_current}object-columns.expect &&
  generate_operate_run +n -A ${standard} -oa 0                      ${file_source} > ${test_current}object-at-0.expect &&
  generate_operate_run +n -A ${standard} -oa 1                      ${file_source} > ${test_current}object-at-1.expect &&
  generate_operate_run +n -A ${standard} -oa 5                      ${file_source} > ${test_current}object-at-5.expect &&
  generate_operate_run +n -A ${standard} -oa 6                      ${file_source} > ${test_current}object-at-6.expect &&
  generate_operate_run +n -A ${standard} -oa 100                    ${file_source} > ${test_current}object-at-100.expect &&
  generate_operate_run +n -A ${standard} -on hi                     ${file_source} > ${test_current}object-name-hi.expect &&
  generate_operate_run +n -A ${standard} -on payload                ${file_source} > ${test_current}object-name-payload.expect &&
  generate_operate_run +n -A ${standard} -on ""                     ${file_source} > ${test_current}object-name-.expect &&
  generate_operate_run +n -A ${standard} -on мир                    ${file_source} > ${test_current}object-name-мир.expect &&
  generate_operate_run +n -A ${standard} -on "привет has space"     ${file_source} > ${test_current}object-name-привет_has_space.expect &&
  generate_operate_run +n -A ${standard} -on a                      ${file_source} > ${test_current}object-name-a.expect &&
  generate_operate_run +n -A ${standard} -onC a                     ${file_source} > ${test_current}object-name-a-columns.expect &&
  generate_operate_run +n -A ${standard} -onCe a                    ${file_source} > ${test_current}object-name-a-columns-empty.expect &&
  generate_operate_run +n -A ${standard} -onO a                     ${file_source} > ${test_current}object-name-a-original.expect &&
  generate_operate_run +n -A ${standard} -onOe a                    ${file_source} > ${test_current}object-name-a-original-empty.expect &&
  generate_operate_run +n -A ${standard} -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run +n -A ${standard} -ont hi                    ${file_source} > ${test_current}object-name-hi-total.expect &&
  generate_operate_run +n -A ${standard} -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run +n -A ${standard} -ont payload               ${file_source} > ${test_current}object-name-payload-total.expect &&
  generate_operate_run +n -A ${standard} -ont a                     ${file_source} > ${test_current}object-name-a-total.expect &&
  generate_operate_run +n -A ${standard} -ont ""                    ${file_source} > ${test_current}object-name--total.expect &&
  generate_operate_run +n -A ${standard} -ont мир                   ${file_source} > ${test_current}object-name-мир-total.expect &&
  generate_operate_run +n -A ${standard} -ont "привет has space"    ${file_source} > ${test_current}object-name-привет_has_space-total.expect &&
  generate_operate_run +n -A ${standard} -ons hi 0                  ${file_source} > ${test_current}object-name-hi-select-0.expect &&
  generate_operate_run +n -A ${standard} -ons hi 1                  ${file_source} > ${test_current}object-name-hi-select-1.expect &&
  generate_operate_run +n -A ${standard} -onse hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-empty.expect &&
  generate_operate_run +n -A ${standard} -onse hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-empty.expect &&
  generate_operate_run +n -A ${standard} -ons payload 0             ${file_source} > ${test_current}object-name-payload-select-0.expect &&
  generate_operate_run +n -A ${standard} -ons payload 1             ${file_source} > ${test_current}object-name-payload-select-1.expect &&
  generate_operate_run +n -A ${standard} -ons a 0                   ${file_source} > ${test_current}object-name-a-select-0.expect &&
  generate_operate_run +n -A ${standard} -ons a 1                   ${file_source} > ${test_current}object-name-a-select-1.expect &&
  generate_operate_run +n -A ${standard} -ons a 2                   ${file_source} > ${test_current}object-name-a-select-2.expect &&
  generate_operate_run +n -A ${standard} -ons a 5                   ${file_source} > ${test_current}object-name-a-select-5.expect &&
  generate_operate_run +n -A ${standard} -ons a 6                   ${file_source} > ${test_current}object-name-a-select-6.expect &&
  generate_operate_run +n -A ${standard} -ons a 100                 ${file_source} > ${test_current}object-name-a-select-100.expect &&
  generate_operate_run +n -A ${standard} -ons "" 0                  ${file_source} > ${test_current}object-name--select-0.expect &&
  generate_operate_run +n -A ${standard} -ons "" 1                  ${file_source} > ${test_current}object-name--select-1.expect &&
  generate_operate_run +n -A ${standard} -ons мир 0                 ${file_source} > ${test_current}object-name-мир-select-0.expect &&
  generate_operate_run +n -A ${standard} -ons мир 1                 ${file_source} > ${test_current}object-name-мир-select-1.expect &&
  generate_operate_run +n -A ${standard} -ons привет 0              ${file_source} > ${test_current}object-name-привет_has_space-select-0.expect &&
  generate_operate_run +n -A ${standard} -ons привет 1              ${file_source} > ${test_current}object-name-привет_has_space-select-1.expect &&
  generate_operate_run +n -A ${standard} -onset hi 0                ${file_source} > ${test_current}object-name-hi-select-0-empty-total.expect &&
  generate_operate_run +n -A ${standard} -onset hi 1                ${file_source} > ${test_current}object-name-hi-select-1-empty-total.expect &&
  generate_operate_run +n -A ${standard} -onst hi 0                 ${file_source} > ${test_current}object-name-hi-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst hi 1                 ${file_source} > ${test_current}object-name-hi-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -onst payload 0            ${file_source} > ${test_current}object-name-payload-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst payload 1            ${file_source} > ${test_current}object-name-payload-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 0                  ${file_source} > ${test_current}object-name-a-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 1                  ${file_source} > ${test_current}object-name-a-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 2                  ${file_source} > ${test_current}object-name-a-select-2-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 5                  ${file_source} > ${test_current}object-name-a-select-5-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 6                  ${file_source} > ${test_current}object-name-a-select-6-total.expect &&
  generate_operate_run +n -A ${standard} -onst a 100                ${file_source} > ${test_current}object-name-a-select-100-total.expect &&
  generate_operate_run +n -A ${standard} -onst "" 0                 ${file_source} > ${test_current}object-name--select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst "" 1                 ${file_source} > ${test_current}object-name--select-1-total.expect &&
  generate_operate_run +n -A ${standard} -onst мир 0                ${file_source} > ${test_current}object-name-мир-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst мир 1                ${file_source} > ${test_current}object-name-мир-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -onst "привет has space" 0 ${file_source} > ${test_current}object-name-привет_has_space-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -onst "привет has space" 1 ${file_source} > ${test_current}object-name-привет_has_space-select-1-total.expect &&

  generate_operate_run +n -A ${standard} -oc                         ${file_source} > ${test_current}object_and_content.expect &&
  generate_operate_run +n -A ${standard} -ocO                        ${file_source} > ${test_current}object_and_content-original.expect &&
  generate_operate_run +n -A ${standard} -oct                        ${file_source} > ${test_current}object_and_content-total.expect &&
  generate_operate_run +n -A ${standard} -ocT                        ${file_source} > ${test_current}object_and_content-trim.expect &&
  generate_operate_run +n -A ${standard} -ocl 0                      ${file_source} > ${test_current}object_and_content-line-0.expect &&
  generate_operate_run +n -A ${standard} -ocl 1                      ${file_source} > ${test_current}object_and_content-line-1.expect &&
  generate_operate_run +n -A ${standard} -ocl 5                      ${file_source} > ${test_current}object_and_content-line-5.expect &&
  generate_operate_run +n -A ${standard} -ocl 6                      ${file_source} > ${test_current}object_and_content-line-6.expect &&
  generate_operate_run +n -A ${standard} -ocl 100                    ${file_source} > ${test_current}object_and_content-line-100.expect &&
  generate_operate_run +n -A ${standard} -oclO 0                     ${file_source} > ${test_current}object_and_content-line-0-original.expect &&
  generate_operate_run +n -A ${standard} -oclO 1                     ${file_source} > ${test_current}object_and_content-line-1-original.expect &&
  generate_operate_run +n -A ${standard} -oclO 5                     ${file_source} > ${test_current}object_and_content-line-5-original.expect &&
  generate_operate_run +n -A ${standard} -oclO 6                     ${file_source} > ${test_current}object_and_content-line-6-original.expect &&
  generate_operate_run +n -A ${standard} -oclO 100                   ${file_source} > ${test_current}object_and_content-line-100-original.expect &&
  generate_operate_run +n -A ${standard} -ocs 0                      ${file_source} > ${test_current}object_and_content-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocs 1                      ${file_source} > ${test_current}object_and_content-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocs 5                      ${file_source} > ${test_current}object_and_content-select-5.expect &&
  generate_operate_run +n -A ${standard} -ocs 6                      ${file_source} > ${test_current}object_and_content-select-6.expect &&
  generate_operate_run +n -A ${standard} -ocs 100                    ${file_source} > ${test_current}object_and_content-select-100.expect &&
  generate_operate_run +n -A ${standard} -ocsO 0                     ${file_source} > ${test_current}object_and_content-select-0-original.expect &&
  generate_operate_run +n -A ${standard} -ocsO 1                     ${file_source} > ${test_current}object_and_content-select-1-original.expect &&
  generate_operate_run +n -A ${standard} -ocsO 5                     ${file_source} > ${test_current}object_and_content-select-5-original.expect &&
  generate_operate_run +n -A ${standard} -ocsO 6                     ${file_source} > ${test_current}object_and_content-select-6-original.expect &&
  generate_operate_run +n -A ${standard} -ocsO 100                   ${file_source} > ${test_current}object_and_content-select-100-original.expect &&
  generate_operate_run +n -A ${standard} -ocC                        ${file_source} > ${test_current}object_and_content-columns.expect &&
  generate_operate_run +n -A ${standard} -ocCe                       ${file_source} > ${test_current}object_and_content-columns-empty.expect &&
  generate_operate_run +n -A ${standard} -oca 0                      ${file_source} > ${test_current}object_and_content-at-0.expect &&
  generate_operate_run +n -A ${standard} -oca 1                      ${file_source} > ${test_current}object_and_content-at-1.expect &&
  generate_operate_run +n -A ${standard} -oca 5                      ${file_source} > ${test_current}object_and_content-at-5.expect &&
  generate_operate_run +n -A ${standard} -oca 6                      ${file_source} > ${test_current}object_and_content-at-6.expect &&
  generate_operate_run +n -A ${standard} -oca 100                    ${file_source} > ${test_current}object_and_content-at-100.expect &&
  generate_operate_run +n -A ${standard} -ocn hi                     ${file_source} > ${test_current}object_and_content-name-hi.expect &&
  generate_operate_run +n -A ${standard} -ocn payload                ${file_source} > ${test_current}object_and_content-name-payload.expect &&
  generate_operate_run +n -A ${standard} -ocn a                      ${file_source} > ${test_current}object_and_content-name-a.expect &&
  generate_operate_run +n -A ${standard} -ocnC a                     ${file_source} > ${test_current}object_and_content-name-a-columns.expect &&
  generate_operate_run +n -A ${standard} -ocnCe a                    ${file_source} > ${test_current}object_and_content-name-a-columns-empty.expect &&
  generate_operate_run +n -A ${standard} -ocnO a                     ${file_source} > ${test_current}object_and_content-name-a-original.expect &&
  generate_operate_run +n -A ${standard} -ocnOe a                    ${file_source} > ${test_current}object_and_content-name-a-original-empty.expect &&
  generate_operate_run +n -A ${standard} -ocn ""                     ${file_source} > ${test_current}object_and_content-name-.expect &&
  generate_operate_run +n -A ${standard} -ocn мир                    ${file_source} > ${test_current}object_and_content-name-мир.expect &&
  generate_operate_run +n -A ${standard} -ocn привет                 ${file_source} > ${test_current}object_and_content-name-привет.expect &&
  generate_operate_run +n -A ${standard} -ocnt hi                    ${file_source} > ${test_current}object_and_content-name-hi-total.expect &&
  generate_operate_run +n -A ${standard} -ocnt payload               ${file_source} > ${test_current}object_and_content-name-payload-total.expect &&
  generate_operate_run +n -A ${standard} -ocnt a                     ${file_source} > ${test_current}object_and_content-name-a-total.expect &&
  generate_operate_run +n -A ${standard} -ocnt ""                    ${file_source} > ${test_current}object_and_content-name--total.expect &&
  generate_operate_run +n -A ${standard} -ocnt мир                   ${file_source} > ${test_current}object_and_content-name-мир-total.expect &&
  generate_operate_run +n -A ${standard} -ocnt "привет has space"    ${file_source} > ${test_current}object_and_content-name-привет_has_space-total.expect &&
  generate_operate_run +n -A ${standard} -ocns hi 0                  ${file_source} > ${test_current}object_and_content-name-hi-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns hi 1                  ${file_source} > ${test_current}object_and_content-name-hi-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocnse hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty.expect &&
  generate_operate_run +n -A ${standard} -ocnse hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty.expect &&
  generate_operate_run +n -A ${standard} -ocns payload 0             ${file_source} > ${test_current}object_and_content-name-payload-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns payload 1             ${file_source} > ${test_current}object_and_content-name-payload-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocns a 0                   ${file_source} > ${test_current}object_and_content-name-a-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns a 1                   ${file_source} > ${test_current}object_and_content-name-a-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocns a 2                   ${file_source} > ${test_current}object_and_content-name-a-select-2.expect &&
  generate_operate_run +n -A ${standard} -ocns a 5                   ${file_source} > ${test_current}object_and_content-name-a-select-5.expect &&
  generate_operate_run +n -A ${standard} -ocns a 6                   ${file_source} > ${test_current}object_and_content-name-a-select-6.expect &&
  generate_operate_run +n -A ${standard} -ocns a 100                 ${file_source} > ${test_current}object_and_content-name-a-select-100.expect &&
  generate_operate_run +n -A ${standard} -ocns "" 0                  ${file_source} > ${test_current}object_and_content-name--select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns "" 1                  ${file_source} > ${test_current}object_and_content-name--select-1.expect &&
  generate_operate_run +n -A ${standard} -ocns мир 0                 ${file_source} > ${test_current}object_and_content-name-мир-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns мир 1                 ${file_source} > ${test_current}object_and_content-name-мир-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocns "привет has space" 0  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0.expect &&
  generate_operate_run +n -A ${standard} -ocns "привет has space" 1  ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1.expect &&
  generate_operate_run +n -A ${standard} -ocnset hi 0                ${file_source} > ${test_current}object_and_content-name-hi-select-0-empty-total.expect &&
  generate_operate_run +n -A ${standard} -ocnset hi 1                ${file_source} > ${test_current}object_and_content-name-hi-select-1-empty-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst hi 0                 ${file_source} > ${test_current}object_and_content-name-hi-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst hi 1                 ${file_source} > ${test_current}object_and_content-name-hi-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst payload 0            ${file_source} > ${test_current}object_and_content-name-payload-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst payload 1            ${file_source} > ${test_current}object_and_content-name-payload-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 0                  ${file_source} > ${test_current}object_and_content-name-a-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 1                  ${file_source} > ${test_current}object_and_content-name-a-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 2                  ${file_source} > ${test_current}object_and_content-name-a-select-2-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 5                  ${file_source} > ${test_current}object_and_content-name-a-select-5-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 6                  ${file_source} > ${test_current}object_and_content-name-a-select-6-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst a 100                ${file_source} > ${test_current}object_and_content-name-a-select-100-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst "" 0                 ${file_source} > ${test_current}object_and_content-name--select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst "" 1                 ${file_source} > ${test_current}object_and_content-name--select-1-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst мир 0                ${file_source} > ${test_current}object_and_content-name-мир-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst мир 1                ${file_source} > ${test_current}object_and_content-name-мир-select-1-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst "привет has space" 0 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-0-total.expect &&
  generate_operate_run +n -A ${standard} -ocnst "привет has space" 1 ${file_source} > ${test_current}object_and_content-name-привет_has_space-select-1-total.expect &&

  generate_operate_test_standard_special_cases &&

  echo "Generation Complete"

  if [[ $? -ne 0 ]] ; then
    let failure=1

    echo
    echo "Failure: ${last_command}"
    echo
  fi

  return $failure
}

generate_operate_test_standard_special_cases() {

  if [[ ${standard} == "0008" ]] ; then
    generate_operate_test_standard_special_cases_0008

    if [[ $? -ne 0 ]] ; then
      let failure=1
    fi

    return $failure
  fi

  return 0
}

generate_operate_test_standard_special_cases_0008() {

  if [[ ${test_base} == "test-0004-nested_simple" ]] ; then
    generate_operate_run_0008 +n -c -d 0                     ${file_source} > ${test_current}content-depth_0.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1                ${file_source} > ${test_current}content-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ct -d 0                     ${file_source} > ${test_current}content-total-depth_0.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1                ${file_source} > ${test_current}content-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 0                     ${file_source} > ${test_current}content-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1                ${file_source} > ${test_current}content-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 0                     ${file_source} > ${test_current}content-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1                ${file_source} > ${test_current}content-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -d 0 -a 0                     ${file_source} > ${test_current}content-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ct -d 0 -a 0                     ${file_source} > ${test_current}content-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0                     ${file_source} > ${test_current}content-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0                     ${file_source} > ${test_current}content-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}content-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 0                     ${file_source} > ${test_current}object-depth_0.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1                ${file_source} > ${test_current}object-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ot -d 0                     ${file_source} > ${test_current}object-total-depth_0.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1                ${file_source} > ${test_current}object-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 0                     ${file_source} > ${test_current}object-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1                ${file_source} > ${test_current}object-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 0                     ${file_source} > ${test_current}object-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1                ${file_source} > ${test_current}object-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 0 -a 0                     ${file_source} > ${test_current}object-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -ot -d 0 -a 0                     ${file_source} > ${test_current}object-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0                     ${file_source} > ${test_current}object-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0                     ${file_source} > ${test_current}object-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -d 0                     ${file_source} > ${test_current}object_content-depth_0.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1                ${file_source} > ${test_current}object_content-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oct -d 0                     ${file_source} > ${test_current}object_content-total-depth_0.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1                ${file_source} > ${test_current}object_content-total-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-total-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 0                     ${file_source} > ${test_current}object_content-line_0-depth_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1                ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_0-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 0                     ${file_source} > ${test_current}object_content-line_1-depth_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1                ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_1-depth_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -d 0 -a 0                     ${file_source} > ${test_current}object_content-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oct -d 0 -a 0                     ${file_source} > ${test_current}object_content-total-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-total-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0                     ${file_source} > ${test_current}object_content-line_0-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_0-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0                     ${file_source} > ${test_current}object_content-line_1-depth_0-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1                ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2           ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 0 -a 0 -d 1 -d 2 -d 3 -d 4 ${file_source} > ${test_current}object_content-line_1-depth_0-at_0-depth_1-depth_2-depth_3-depth_4.expect &&

    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ot -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ot -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object-line_1-name_depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object-line_1-name_depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -o -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -ct -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -ct -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -c -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}content-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_1-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oct -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-total-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oct -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-total-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_0-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 0 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-line_0-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect &&

    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name                ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name                ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets               ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 1           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 1          ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_1.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 2           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2           ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 2          ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_2.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 1 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 1 -d 3     ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_1-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_1_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_1_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3      ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 1 -n depth_1_set_3_same_name -a 2 -d 3 -a 0 ${file_source} > ${test_current}object_content-line_1-depth_1-name_depth_1_set_3_same_name-at_2-depth_3-at_0.expect &&
    generate_operate_run_0008 +n -oc -l 1 -d 2 -n depth_2_same_across_sets -a 2 -d 3     ${file_source} > ${test_current}object_content-line_1-depth_2-name_depth_2_same_across_sets-at_2-depth_3.expect

    if [[ $? -ne 0 ]] ; then
      let failure=1
    fi

    return $failure
  fi

  return 0
}

generate_cleanup() {

  unset generate_main
  unset generate_operate_0000
  unset generate_operate_0001
  unset generate_operate_0002
  unset generate_operate_0003
  unset generate_operate_0008
  unset generate_operate_0008_special_cases
  unset generate_operate_000e
  unset generate_operate_run
  unset generate_operate_run_0000
  unset generate_operate_run_0001
  unset generate_operate_run_0002
  unset generate_operate_run_0003
  unset generate_operate_run_0008
  unset generate_operate_run_000e
  unset generate_operate_test_standard
  unset generate_operate_test_standard_special_cases
  unset generate_operate_test_standard_special_cases_0008
  unset generate_cleanup
}

generate_main "$@"
